@@ -1,4 +1,4 @@
-# Copyright 2010-2012 10gen, Inc.
+# Copyright 2010-2015 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,15 +18,30 @@
 
 class MaxKey(object):
     """MongoDB internal MaxKey type.
+
+    .. versionchanged:: 2.7
+       ``MaxKey`` now implements comparison operators.
     """
 
+    _type_marker = 127
+
     def __eq__(self, other):
-        if isinstance(other, MaxKey):
-            return True
-        return NotImplemented
+        return isinstance(other, MaxKey)
 
     def __ne__(self, other):
         return not self == other
 
+    def __le__(self, other):
+        return isinstance(other, MaxKey)
+    
+    def __lt__(self, dummy):
+        return False
+
+    def __ge__(self, dummy):
+        return True
+    
+    def __gt__(self, other):
+        return not isinstance(other, MaxKey)
+
     def __repr__(self):
         return "MaxKey()"
