@@ -27,7 +27,7 @@ master_doc = 'index'
 
 # General information about the project.
 project = u'PyMongo'
-copyright = u'2008 - 2014, MongoDB, Inc.'
+copyright = u'2008 - 2015, MongoDB, Inc.'
 
 # The version info for the project you're documenting, acts as replacement for
 # |version| and |release|, also used in various other places throughout the
@@ -78,11 +78,14 @@ db = client.doctest_test
 
 # -- Options for HTML output ---------------------------------------------------
 
-# The theme to use for HTML and HTML Help pages.  Major themes that come with
-# Sphinx are currently 'default' and 'sphinxdoc'.
-html_theme = 'default'
+# Theme gratefully vendored from CPython source.
+html_theme = "pydoctheme"
+html_theme_path = ["."]
 html_theme_options = {'collapsiblesidebar': True}
 
+# Additional static files.
+html_static_path = ['static']
+
 # The name for this set of Sphinx documents.  If None, it defaults to
 # "<project> v<release> documentation".
 #html_title = None
