@@ -870,8 +870,11 @@ class Collection(common.BaseObject):
             error.
           - `oplog_replay` (optional): If True, set the oplogReplay query
             flag.
-          - `modifiers` (optional): A dict specifying the MongoDB query
-            modifiers that should be used for this query.
+          - `modifiers` (optional): A dict specifying the MongoDB `query
+            modifiers`_ that should be used for this query. For example::
+
+              >>> db.test.find(modifiers={"$maxTimeMS": 500})
+
           - `batch_size` (optional): Limits the number of documents returned in
             a single batch.
           - `manipulate` (optional): **DEPRECATED** - If True (the default),
@@ -918,6 +921,8 @@ class Collection(common.BaseObject):
            The `tag_sets` and `secondary_acceptable_latency_ms` parameters.
 
         .. _PYTHON-500: https://jira.mongodb.org/browse/PYTHON-500
+        .. _query modifiers:
+          http://docs.mongodb.org/manual/reference/operator/query-modifier/
 
         .. mongodoc:: find
         """
@@ -975,6 +980,15 @@ class Collection(common.BaseObject):
         return [CommandCursor(self, cursor['cursor'], sock_info.address)
                 for cursor in result['cursors']]
 
+    def _count(self, cmd):
+        """Internal count helper."""
+        with self._socket_for_reads() as (sock_info, slave_ok):
+            res = self._command(sock_info, cmd, slave_ok,
+                                allowable_errors=["ns missing"])
+        if res.get("errmsg", "") == "ns missing":
+            return 0
+        return int(res["n"])
+
     def count(self, filter=None, **kwargs):
         """Get the number of documents in this collection.
 
@@ -1006,12 +1020,7 @@ class Collection(common.BaseObject):
         if "hint" in kwargs and not isinstance(kwargs["hint"], string_type):
             kwargs["hint"] = helpers._index_document(kwargs["hint"])
         cmd.update(kwargs)
-        with self._socket_for_reads() as (sock_info, slave_ok):
-            res = self._command(sock_info, cmd, slave_ok,
-                                allowable_errors=["ns missing"])
-        if res.get("errmsg", "") == "ns missing":
-            return 0
-        return int(res["n"])
+        return self._count(cmd)
 
     def create_indexes(self, indexes):
         """Create one or more indexes on this collection.
@@ -1255,23 +1264,31 @@ class Collection(common.BaseObject):
 
         .. versionadded:: 3.0
         """
+        codec_options = CodecOptions(SON)
+        coll = self.with_options(codec_options)
         with self._socket_for_primary_reads() as (sock_info, slave_ok):
             if sock_info.max_wire_version > 2:
                 cmd = SON([("listIndexes", self.__name), ("cursor", {})])
                 cursor = self._command(sock_info, cmd, slave_ok,
                                        ReadPreference.PRIMARY,
-                                       CodecOptions(SON))["cursor"]
+                                       codec_options)["cursor"]
+                return CommandCursor(coll, cursor, sock_info.address)
             else:
                 namespace = _UJOIN % (self.__database.name, "system.indexes")
                 res = helpers._first_batch(
                     sock_info, namespace, {"ns": self.__full_name},
-                    0, slave_ok, CodecOptions(SON), ReadPreference.PRIMARY)
+                    0, slave_ok, codec_options, ReadPreference.PRIMARY)
+                data = res["data"]
                 cursor = {
                     "id": res["cursor_id"],
-                    "firstBatch": res["data"],
+                    "firstBatch": data,
                     "ns": namespace,
                 }
-            return CommandCursor(self, cursor, sock_info.address)
+                # Note that a collection can only have 64 indexes, so we don't
+                # technically have to pass len(data) here. There will never be
+                # an OP_GET_MORE call.
+                return CommandCursor(
+                    coll, cursor, sock_info.address, len(data))
 
     def index_information(self):
         """Get information on this collection's indexes.
@@ -1887,6 +1904,7 @@ class Collection(common.BaseObject):
             else:
                 self._update(sock_info, {"_id": to_save["_id"]}, to_save, True,
                              check_keys, False, manipulate, write_concern)
+                return to_save.get("_id")
 
     def insert(self, doc_or_docs, manipulate=True,
                check_keys=True, continue_on_error=False, **kwargs):
