@@ -1,4 +1,4 @@
-# Copyright 2009-2014 MongoDB, Inc.
+# Copyright 2009-2015 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,15 +23,11 @@ except ImportError:
 
 
 class PyMongoError(Exception):
-    """Base class for all PyMongo exceptions.
-
-    .. versionadded:: 1.4
-    """
+    """Base class for all PyMongo exceptions."""
 
 
 class ConnectionFailure(PyMongoError):
-    """Raised when a connection to the database cannot be made or is lost.
-    """
+    """Raised when a connection to the database cannot be made or is lost."""
 
 
 class AutoReconnect(ConnectionFailure):
@@ -43,12 +39,52 @@ class AutoReconnect(ConnectionFailure):
     operations will attempt to open a new connection to the database (and
     will continue to raise this exception until the first successful
     connection is made).
+
+    Subclass of :exc:`~pymongo.errors.ConnectionFailure`.
     """
     def __init__(self, message='', errors=None):
         self.errors = errors or []
         ConnectionFailure.__init__(self, message)
 
 
+class NetworkTimeout(AutoReconnect):
+    """An operation on an open connection exceeded socketTimeoutMS.
+
+    The remaining connections in the pool stay open. In the case of a write
+    operation, you cannot know whether it succeeded or failed.
+
+    Subclass of :exc:`~pymongo.errors.AutoReconnect`.
+    """
+
+
+class NotMasterError(AutoReconnect):
+    """The server responded "not master" or "node is recovering".
+
+    These errors result from a query, write, or command. The operation failed
+    because the client thought it was using the primary but the primary has
+    stepped down, or the client thought it was using a healthy secondary but
+    the secondary is stale and trying to recover.
+
+    The client launches a refresh operation on a background thread, to update
+    its view of the server as soon as possible after throwing this exception.
+
+    Subclass of :exc:`~pymongo.errors.AutoReconnect`.
+    """
+
+
+class ServerSelectionTimeoutError(AutoReconnect):
+    """Thrown when no MongoDB server is available for an operation
+
+    If there is no suitable server for an operation PyMongo tries for
+    ``serverSelectionTimeoutMS`` (default 30 seconds) to find one, then
+    throws this exception. For example, it is thrown after attempting an
+    operation when PyMongo cannot connect to any server, or if you attempt
+    an insert into a replica set that has no primary and does not elect one
+    within the timeout window, or if you attempt to query with a Read
+    Preference that the replica set cannot satisfy.
+    """
+
+
 class ConfigurationError(PyMongoError):
     """Raised when something is incorrectly configured.
     """
@@ -59,9 +95,6 @@ class OperationFailure(PyMongoError):
 
     .. versionadded:: 2.7
        The :attr:`details` attribute.
-
-    .. versionadded:: 1.8
-       The :attr:`code` attribute.
     """
 
     def __init__(self, error, code=None, details=None):
@@ -106,14 +139,21 @@ class ExecutionTimeout(OperationFailure):
     """
 
 
-class TimeoutError(OperationFailure):
-    """DEPRECATED - will be removed in PyMongo 3.0. See WTimeoutError instead.
+class WriteConcernError(OperationFailure):
+    """Base exception type for errors raised due to write concern.
 
-    .. versionadded:: 1.8
+    .. versionadded:: 3.0
     """
 
 
-class WTimeoutError(TimeoutError):
+class WriteError(OperationFailure):
+    """Base exception type for errors raised during write operations.
+
+    .. versionadded:: 3.0
+    """
+
+
+class WTimeoutError(WriteConcernError):
     """Raised when a database operation times out (i.e. wtimeout expires)
     before replication completes.
 
@@ -124,13 +164,8 @@ class WTimeoutError(TimeoutError):
     """
 
 
-class DuplicateKeyError(OperationFailure):
-    """Raised when a safe insert or update fails due to a duplicate key error.
-
-    .. note:: Requires server version **>= 1.3.0**
-
-    .. versionadded:: 1.4
-    """
+class DuplicateKeyError(WriteError):
+    """Raised when an insert or update fails due to a duplicate key error."""
 
 
 class BulkWriteError(OperationFailure):
@@ -144,37 +179,24 @@ class BulkWriteError(OperationFailure):
 
 
 class InvalidOperation(PyMongoError):
-    """Raised when a client attempts to perform an invalid operation.
-    """
+    """Raised when a client attempts to perform an invalid operation."""
 
 
 class InvalidName(PyMongoError):
-    """Raised when an invalid name is used.
-    """
+    """Raised when an invalid name is used."""
 
 
 class CollectionInvalid(PyMongoError):
-    """Raised when collection validation fails.
-    """
+    """Raised when collection validation fails."""
 
 
 class InvalidURI(ConfigurationError):
-    """Raised when trying to parse an invalid mongodb URI.
-
-    .. versionadded:: 1.5
-    """
-
-
-class UnsupportedOption(ConfigurationError):
-    """Exception for unsupported options.
-
-    .. versionadded:: 2.0
-    """
+    """Raised when trying to parse an invalid mongodb URI."""
 
 
 class ExceededMaxWaiters(Exception):
     """Raised when a thread tries to get a connection from a pool and
-    ``max_pool_size * waitQueueMultiple`` threads are already waiting.
+    ``maxPoolSize * waitQueueMultiple`` threads are already waiting.
 
     .. versionadded:: 2.6
     """
