@@ -1,4 +1,4 @@
-# Copyright 2011-2012 10gen, Inc.
+# Copyright 2011-2015 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you
 # may not use this file except in compliance with the License.  You
@@ -15,12 +15,16 @@
 
 """Tools to parse and validate a MongoDB URI."""
 
-from urllib import unquote_plus
+from bson.py3compat import PY3, iteritems, string_type
+
+if PY3:
+    from urllib.parse import unquote_plus
+else:
+    from urllib import unquote_plus
+
+from pymongo.common import validate as _validate
+from pymongo.errors import ConfigurationError, InvalidURI
 
-from pymongo.common import validate
-from pymongo.errors import (ConfigurationError,
-                            InvalidURI,
-                            UnsupportedOption)
 
 SCHEME = 'mongodb://'
 SCHEME_LEN = len(SCHEME)
@@ -100,9 +104,9 @@ def parse_ipv6_literal_host(entity, default_port):
                           specified in entity.
     """
     if entity.find(']') == -1:
-        raise ConfigurationError("an IPv6 address literal must be "
-                                 "enclosed in '[' and ']' according "
-                                 "to RFC 2732.")
+        raise ValueError("an IPv6 address literal must be "
+                         "enclosed in '[' and ']' according "
+                         "to RFC 2732.")
     i = entity.find(']:')
     if i == -1:
         return entity[1:-1], default_port
@@ -127,16 +131,21 @@ def parse_host(entity, default_port=DEFAULT_PORT):
         host, port = parse_ipv6_literal_host(entity, default_port)
     elif entity.find(':') != -1:
         if entity.count(':') > 1:
-            raise ConfigurationError("Reserved characters such as ':' must be "
-                                     "escaped according RFC 2396. An IPv6 "
-                                     "address literal must be enclosed in '[' "
-                                     "and ']' according to RFC 2732.")
+            raise ValueError("Reserved characters such as ':' must be "
+                             "escaped according RFC 2396. An IPv6 "
+                             "address literal must be enclosed in '[' "
+                             "and ']' according to RFC 2732.")
         host, port = host.split(':', 1)
-    if isinstance(port, basestring):
+    if isinstance(port, string_type):
         if not port.isdigit():
-            raise ConfigurationError("Port number must be an integer.")
+            raise ValueError("Port number must be an integer.")
         port = int(port)
-    return host, port
+
+    # Normalize hostname to lowercase, since DNS is case-insensitive:
+    # http://tools.ietf.org/html/rfc4343
+    # This prevents useless rediscovery if "foo.com" is in the seed list but
+    # "FOO.com" is in the ismaster response.
+    return host.lower(), port
 
 
 def validate_options(opts):
@@ -147,23 +156,34 @@ def validate_options(opts):
     :Parameters:
         - `opts`: A dict of MongoDB URI options.
     """
-    normalized = {}
-    for option, value in opts.iteritems():
-        option, value = validate(option, value)
-        # str(option) to ensure that a unicode URI results in plain 'str'
-        # option names. 'normalized' is then suitable to be passed as kwargs
-        # in all Python versions.
-        normalized[str(option)] = value
-    return normalized
+    return dict([_validate(opt, val) for opt, val in iteritems(opts)])
 
 
-def split_options(opts):
+def _parse_options(opts, delim):
+    """Helper method for split_options which creates the options dict.
+    Also handles the creation of a list for the URI tag_sets/
+    readpreferencetags portion."""
+    options = {}
+    for opt in opts.split(delim):
+        key, val = opt.split("=")
+        if key.lower() == 'readpreferencetags':
+            options.setdefault('readpreferencetags', []).append(val)
+        else:
+            # str(option) to ensure that a unicode URI results in plain 'str'
+            # option names. 'normalized' is then suitable to be passed as
+            # kwargs in all Python versions.
+            options[str(key)] = val
+    return options
+
+
+def split_options(opts, validate=True):
     """Takes the options portion of a MongoDB URI, validates each option
-    and returns the options in a dictionary. The option names will be returned
-    lowercase even if camelCase options are used.
+    and returns the options in a dictionary.
 
     :Parameters:
         - `opt`: A string representing MongoDB URI options.
+        - `validate`: If ``True`` (the default), validate and normalize all
+          options.
     """
     and_idx = opts.find("&")
     semi_idx = opts.find(";")
@@ -171,17 +191,19 @@ def split_options(opts):
         if and_idx >= 0 and semi_idx >= 0:
             raise InvalidURI("Can not mix '&' and ';' for option separators.")
         elif and_idx >= 0:
-            options = dict([kv.split("=") for kv in opts.split("&")])
+            options = _parse_options(opts, "&")
         elif semi_idx >= 0:
-            options = dict([kv.split("=") for kv in opts.split(";")])
+            options = _parse_options(opts, ";")
         elif opts.find("=") != -1:
-            options = dict([opts.split("=")])
+            options = _parse_options(opts, None)
         else:
             raise ValueError
     except ValueError:
         raise InvalidURI("MongoDB URI options are key=value pairs.")
 
-    return validate_options(options)
+    if validate:
+        return validate_options(options)
+    return options
 
 
 def split_hosts(hosts, default_port=DEFAULT_PORT):
@@ -195,7 +217,7 @@ def split_hosts(hosts, default_port=DEFAULT_PORT):
     :Parameters:
         - `hosts`: A string of the form host1[:port],host2[:port],...
         - `default_port`: The port number to use when one wasn't specified
-                          for a host.
+          for a host.
     """
     nodes = []
     for entity in hosts.split(','):
@@ -210,7 +232,7 @@ def split_hosts(hosts, default_port=DEFAULT_PORT):
     return nodes
 
 
-def parse_uri(uri, default_port=DEFAULT_PORT):
+def parse_uri(uri, default_port=DEFAULT_PORT, validate=True):
     """Parse and validate a MongoDB URI.
 
     Returns a dict of the form::
@@ -227,7 +249,9 @@ def parse_uri(uri, default_port=DEFAULT_PORT):
     :Parameters:
         - `uri`: The MongoDB URI to parse.
         - `default_port`: The port number to use when one wasn't specified
-                          for a host in the URI.
+          for a host in the URI.
+        - `validate`: If ``True`` (the default), validate and normalize all
+          options.
     """
     if not uri.startswith(SCHEME):
         raise InvalidURI("Invalid URI scheme: URI "
@@ -278,7 +302,7 @@ def parse_uri(uri, default_port=DEFAULT_PORT):
                 dbase, collection = dbase.split('.', 1)
 
         if opts:
-            options = split_options(opts)
+            options = split_options(opts, validate)
 
     return {
         'nodelist': nodes,
@@ -295,7 +319,6 @@ if __name__ == '__main__':
     import sys
     try:
         pprint.pprint(parse_uri(sys.argv[1]))
-    except (InvalidURI, UnsupportedOption), e:
-        print e
+    except InvalidURI as e:
+        print(e)
     sys.exit(0)
-
