@@ -121,6 +121,10 @@ class ClientUnitTest(unittest.TestCase):
 
         self.assertRaises(ConfigurationError, MongoClient, [])
 
+    def test_max_pool_size_zero(self):
+        with self.assertRaises(ValueError):
+            MongoClient(maxPoolSize=0)
+
     def test_get_db(self):
         def make_db(base, name):
             return base[name]
