@@ -1,4 +1,4 @@
-# Copyright 2009-2014 MongoDB, Inc.
+# Copyright 2009-2015 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,11 +14,12 @@
 
 """Tests for the Code wrapper."""
 
-import unittest
 import sys
 sys.path[0:0] = [""]
 
 from bson.code import Code
+from bson.py3compat import u
+from test import unittest
 
 
 class TestCode(unittest.TestCase):
@@ -26,11 +27,11 @@ class TestCode(unittest.TestCase):
         self.assertRaises(TypeError, Code, 5)
         self.assertRaises(TypeError, Code, None)
         self.assertRaises(TypeError, Code, "aoeu", 5)
-        self.assertRaises(TypeError, Code, u"aoeu", 5)
+        self.assertRaises(TypeError, Code, u("aoeu"), 5)
         self.assertTrue(Code("aoeu"))
-        self.assertTrue(Code(u"aoeu"))
+        self.assertTrue(Code(u("aoeu")))
         self.assertTrue(Code("aoeu", {}))
-        self.assertTrue(Code(u"aoeu", {}))
+        self.assertTrue(Code(u("aoeu"), {}))
 
     def test_read_only(self):
         c = Code("blah")
