@@ -634,11 +634,16 @@ class TestCursor(IntegrationTest):
         if client_context.version.at_least(2, 6, 0):
             # Count supports hint
             self.assertEqual(0, collection.find({'i': 1}).hint("x_1").count())
+            self.assertEqual(
+                0, collection.find({'i': 1}).hint([("x", 1)]).count())
         else:
             # Hint is ignored
             self.assertEqual(1, collection.find({'i': 1}).hint("x_1").count())
+            self.assertEqual(
+                1, collection.find({'i': 1}).hint([("x", 1)]).count())
 
         self.assertEqual(2, collection.find().hint("x_1").count())
+        self.assertEqual(2, collection.find().hint([("x", 1)]).count())
 
     def test_where(self):
         db = self.db
@@ -1106,5 +1111,34 @@ class TestCursor(IntegrationTest):
         docs.extend(ccursor)
         self.assertEqual(len(docs), 200)
 
+    def test_modifiers(self):
+        cur = self.db.test.find()
+        self.assertTrue('$query' not in cur._Cursor__query_spec())
+        cur = self.db.test.find().comment("testing").max_time_ms(500)
+        self.assertTrue('$query' in cur._Cursor__query_spec())
+        self.assertEqual(cur._Cursor__query_spec()["$comment"], "testing")
+        self.assertEqual(cur._Cursor__query_spec()["$maxTimeMS"], 500)
+        cur = self.db.test.find(
+            modifiers={"$maxTimeMS": 500, "$comment": "testing"})
+        self.assertTrue('$query' in cur._Cursor__query_spec())
+        self.assertEqual(cur._Cursor__query_spec()["$comment"], "testing")
+        self.assertEqual(cur._Cursor__query_spec()["$maxTimeMS"], 500)
+
+    def test_alive(self):
+        self.db.test.delete_many({})
+        self.db.test.insert_many([{} for _ in range(3)])
+        self.addCleanup(self.db.test.delete_many, {})
+        cursor = self.db.test.find().batch_size(2)
+        n = 0
+        while True:
+            cursor.next()
+            n += 1
+            if 3 == n:
+                self.assertFalse(cursor.alive)
+                break
+
+            self.assertTrue(cursor.alive)
+
+
 if __name__ == "__main__":
     unittest.main()
