@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2009-2014 MongoDB, Inc.
+# Copyright 2009-2015 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,55 +19,90 @@
 
 import datetime
 import sys
-import unittest
 sys.path[0:0] = [""]
 
-from nose.plugins.skip import SkipTest
-
 from bson.objectid import ObjectId
-from bson.py3compat import b, StringIO
+from bson.py3compat import u, StringIO
 from gridfs import GridFS
 from gridfs.grid_file import (DEFAULT_CHUNK_SIZE,
                               _SEEK_CUR,
                               _SEEK_END,
                               GridIn,
-                              GridFile,
                               GridOut,
                               GridOutCursor)
-from gridfs.errors import (NoFile,
-                           UnsupportedAPI)
+from gridfs.errors import NoFile
 from pymongo import MongoClient
-from pymongo.errors import ConnectionFailure
-from test.test_client import get_client
-from test import qcheck, skip_restricted_localhost
+from pymongo.errors import ConfigurationError, ConnectionFailure
+from test import (IntegrationTest,
+                  host,
+                  port,
+                  unittest,
+                  qcheck)
+from test.utils import rs_or_single_client
 
 
-setUpModule = skip_restricted_localhost
+class TestGridFileNoConnect(unittest.TestCase):
+
+    @classmethod
+    def setUpClass(cls):
+        client = MongoClient(host, port, connect=False)
+        cls.db = client.pymongo_test
+
+    def test_grid_in_custom_opts(self):
+        self.assertRaises(TypeError, GridIn, "foo")
+
+        a = GridIn(self.db.fs, _id=5, filename="my_file",
+                   contentType="text/html", chunkSize=1000, aliases=["foo"],
+                   metadata={"foo": 1, "bar": 2}, bar=3, baz="hello")
+
+        self.assertEqual(5, a._id)
+        self.assertEqual("my_file", a.filename)
+        self.assertEqual("my_file", a.name)
+        self.assertEqual("text/html", a.content_type)
+        self.assertEqual(1000, a.chunk_size)
+        self.assertEqual(["foo"], a.aliases)
+        self.assertEqual({"foo": 1, "bar": 2}, a.metadata)
+        self.assertEqual(3, a.bar)
+        self.assertEqual("hello", a.baz)
+        self.assertRaises(AttributeError, getattr, a, "mike")
+
+        b = GridIn(self.db.fs,
+                   content_type="text/html", chunk_size=1000, baz=100)
+        self.assertEqual("text/html", b.content_type)
+        self.assertEqual(1000, b.chunk_size)
+        self.assertEqual(100, b.baz)
+
+    def test_grid_out_cursor_options(self):
+        self.assertRaises(TypeError, GridOutCursor.__init__, self.db.fs, {},
+                          projection={"filename": 1})
+
+        cursor = GridOutCursor(self.db.fs, {})
+        cursor_clone = cursor.clone()
+        self.assertEqual(cursor_clone.__dict__, cursor.__dict__)
+
+        self.assertRaises(NotImplementedError, cursor.add_option, 0)
+        self.assertRaises(NotImplementedError, cursor.remove_option, 0)
 
 
-class TestGridFile(unittest.TestCase):
+class TestGridFile(IntegrationTest):
 
     def setUp(self):
-        self.db = get_client().pymongo_test
-        self.db.fs.files.remove({})
-        self.db.fs.chunks.remove({})
-
-    def tearDown(self):
-        self.db = None
+        self.db.drop_collection('fs.files')
+        self.db.drop_collection('fs.chunks')
 
     def test_basic(self):
         f = GridIn(self.db.fs, filename="test")
-        f.write(b("hello world"))
+        f.write(b"hello world")
         f.close()
         self.assertEqual(1, self.db.fs.files.find().count())
         self.assertEqual(1, self.db.fs.chunks.find().count())
 
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual(b("hello world"), g.read())
+        self.assertEqual(b"hello world", g.read())
 
         # make sure it's still there...
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual(b("hello world"), g.read())
+        self.assertEqual(b"hello world", g.read())
 
         f = GridIn(self.db.fs, filename="test")
         f.close()
@@ -75,37 +110,34 @@ class TestGridFile(unittest.TestCase):
         self.assertEqual(1, self.db.fs.chunks.find().count())
 
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual(b(""), g.read())
+        self.assertEqual(b"", g.read())
 
         # test that reading 0 returns proper type
-        self.assertEqual(b(""), g.read(0))
+        self.assertEqual(b"", g.read(0))
 
     def test_md5(self):
         f = GridIn(self.db.fs)
-        f.write(b("hello world\n"))
+        f.write(b"hello world\n")
         f.close()
         self.assertEqual("6f5902ac237024bdd0c176cb93063dc4", f.md5)
 
     def test_alternate_collection(self):
-        self.db.alt.files.remove({})
-        self.db.alt.chunks.remove({})
+        self.db.alt.files.delete_many({})
+        self.db.alt.chunks.delete_many({})
 
         f = GridIn(self.db.alt)
-        f.write(b("hello world"))
+        f.write(b"hello world")
         f.close()
 
         self.assertEqual(1, self.db.alt.files.find().count())
         self.assertEqual(1, self.db.alt.chunks.find().count())
 
         g = GridOut(self.db.alt, f._id)
-        self.assertEqual(b("hello world"), g.read())
+        self.assertEqual(b"hello world", g.read())
 
         # test that md5 still works...
         self.assertEqual("5eb63bbbe01eeed093cb22bb8f5acdc3", g.md5)
 
-    def test_grid_file(self):
-        self.assertRaises(UnsupportedAPI, GridFile)
-
     def test_grid_in_default_opts(self):
         self.assertRaises(TypeError, GridIn, "foo")
 
@@ -180,34 +212,12 @@ class TestGridFile(unittest.TestCase):
         self.assertEqual(a.aliases, b.aliases)
         self.assertEqual(a.forty_two, b.forty_two)
 
-    def test_grid_in_custom_opts(self):
-        self.assertRaises(TypeError, GridIn, "foo")
-
-        a = GridIn(self.db.fs, _id=5, filename="my_file",
-                   contentType="text/html", chunkSize=1000, aliases=["foo"],
-                   metadata={"foo": 1, "bar": 2}, bar=3, baz="hello")
-
-        self.assertEqual(5, a._id)
-        self.assertEqual("my_file", a.filename)
-        self.assertEqual("my_file", a.name)
-        self.assertEqual("text/html", a.content_type)
-        self.assertEqual(1000, a.chunk_size)
-        self.assertEqual(["foo"], a.aliases)
-        self.assertEqual({"foo": 1, "bar": 2}, a.metadata)
-        self.assertEqual(3, a.bar)
-        self.assertEqual("hello", a.baz)
-        self.assertRaises(AttributeError, getattr, a, "mike")
-
-        b = GridIn(self.db.fs,
-                   content_type="text/html", chunk_size=1000, baz=100)
-        self.assertEqual("text/html", b.content_type)
-        self.assertEqual(1000, b.chunk_size)
-        self.assertEqual(100, b.baz)
-
     def test_grid_out_default_opts(self):
         self.assertRaises(TypeError, GridOut, "foo")
 
-        self.assertRaises(NoFile, GridOut, self.db.fs, 5)
+        gout = GridOut(self.db.fs, 5)
+        with self.assertRaises(NoFile):
+            gout.name
 
         a = GridIn(self.db.fs)
         a.close()
@@ -233,7 +243,7 @@ class TestGridFile(unittest.TestCase):
         one = GridIn(self.db.fs, _id=5, filename="my_file",
                    contentType="text/html", chunkSize=1000, aliases=["foo"],
                    metadata={"foo": 1, "bar": 2}, bar=3, baz="hello")
-        one.write(b("hello world"))
+        one.write(b"hello world")
         one.close()
 
         two = GridOut(self.db.fs, 5)
@@ -256,20 +266,22 @@ class TestGridFile(unittest.TestCase):
 
     def test_grid_out_file_document(self):
         one = GridIn(self.db.fs)
-        one.write(b("foo bar"))
+        one.write(b"foo bar")
         one.close()
 
         two = GridOut(self.db.fs, file_document=self.db.fs.files.find_one())
-        self.assertEqual(b("foo bar"), two.read())
+        self.assertEqual(b"foo bar", two.read())
 
         three = GridOut(self.db.fs, 5, file_document=self.db.fs.files.find_one())
-        self.assertEqual(b("foo bar"), three.read())
+        self.assertEqual(b"foo bar", three.read())
 
-        self.assertRaises(NoFile, GridOut, self.db.fs, file_document={})
+        four = GridOut(self.db.fs, file_document={})
+        with self.assertRaises(NoFile):
+            four.name
 
     def test_write_file_like(self):
         one = GridIn(self.db.fs)
-        one.write(b("hello world"))
+        one.write(b"hello world")
         one.close()
 
         two = GridOut(self.db.fs, one._id)
@@ -279,23 +291,23 @@ class TestGridFile(unittest.TestCase):
         three.close()
 
         four = GridOut(self.db.fs, three._id)
-        self.assertEqual(b("hello world"), four.read())
+        self.assertEqual(b"hello world", four.read())
 
         five = GridIn(self.db.fs, chunk_size=2)
-        five.write(b("hello"))
-        buffer = StringIO(b(" world"))
+        five.write(b"hello")
+        buffer = StringIO(b" world")
         five.write(buffer)
-        five.write(b(" and mongodb"))
+        five.write(b" and mongodb")
         five.close()
-        self.assertEqual(b("hello world and mongodb"),
+        self.assertEqual(b"hello world and mongodb",
                          GridOut(self.db.fs, five._id).read())
 
     def test_write_lines(self):
         a = GridIn(self.db.fs)
-        a.writelines([b("hello "), b("world")])
+        a.writelines([b"hello ", b"world"])
         a.close()
 
-        self.assertEqual(b("hello world"), GridOut(self.db.fs, a._id).read())
+        self.assertEqual(b"hello world", GridOut(self.db.fs, a._id).read())
 
     def test_close(self):
         f = GridIn(self.db.fs)
@@ -304,7 +316,7 @@ class TestGridFile(unittest.TestCase):
         f.close()
 
     def test_multi_chunk_file(self):
-        random_string = b('a') * (DEFAULT_CHUNK_SIZE + 1000)
+        random_string = b'a' * (DEFAULT_CHUNK_SIZE + 1000)
 
         f = GridIn(self.db.fs)
         f.write(random_string)
@@ -343,31 +355,31 @@ class TestGridFile(unittest.TestCase):
 
     def test_seek(self):
         f = GridIn(self.db.fs, chunkSize=3)
-        f.write(b("hello world"))
+        f.write(b"hello world")
         f.close()
 
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual(b("hello world"), g.read())
+        self.assertEqual(b"hello world", g.read())
         g.seek(0)
-        self.assertEqual(b("hello world"), g.read())
+        self.assertEqual(b"hello world", g.read())
         g.seek(1)
-        self.assertEqual(b("ello world"), g.read())
+        self.assertEqual(b"ello world", g.read())
         self.assertRaises(IOError, g.seek, -1)
 
         g.seek(-3, _SEEK_END)
-        self.assertEqual(b("rld"), g.read())
+        self.assertEqual(b"rld", g.read())
         g.seek(0, _SEEK_END)
-        self.assertEqual(b(""), g.read())
+        self.assertEqual(b"", g.read())
         self.assertRaises(IOError, g.seek, -100, _SEEK_END)
 
         g.seek(3)
         g.seek(3, _SEEK_CUR)
-        self.assertEqual(b("world"), g.read())
+        self.assertEqual(b"world", g.read())
         self.assertRaises(IOError, g.seek, -100, _SEEK_CUR)
 
     def test_tell(self):
         f = GridIn(self.db.fs, chunkSize=3)
-        f.write(b("hello world"))
+        f.write(b"hello world")
         f.close()
 
         g = GridOut(self.db.fs, f._id)
@@ -383,21 +395,21 @@ class TestGridFile(unittest.TestCase):
 
     def test_multiple_reads(self):
         f = GridIn(self.db.fs, chunkSize=3)
-        f.write(b("hello world"))
+        f.write(b"hello world")
         f.close()
 
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual(b("he"), g.read(2))
-        self.assertEqual(b("ll"), g.read(2))
-        self.assertEqual(b("o "), g.read(2))
-        self.assertEqual(b("wo"), g.read(2))
-        self.assertEqual(b("rl"), g.read(2))
-        self.assertEqual(b("d"), g.read(2))
-        self.assertEqual(b(""), g.read(2))
+        self.assertEqual(b"he", g.read(2))
+        self.assertEqual(b"ll", g.read(2))
+        self.assertEqual(b"o ", g.read(2))
+        self.assertEqual(b"wo", g.read(2))
+        self.assertEqual(b"rl", g.read(2))
+        self.assertEqual(b"d", g.read(2))
+        self.assertEqual(b"", g.read(2))
 
     def test_readline(self):
         f = GridIn(self.db.fs, chunkSize=5)
-        f.write(b("""Hello world,
+        f.write((b"""Hello world,
 How are you?
 Hope all is well.
 Bye"""))
@@ -405,27 +417,27 @@ Bye"""))
 
         # Try read(), then readline().
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual(b("H"), g.read(1))
-        self.assertEqual(b("ello world,\n"), g.readline())
-        self.assertEqual(b("How a"), g.readline(5))
-        self.assertEqual(b(""), g.readline(0))
-        self.assertEqual(b("re you?\n"), g.readline())
-        self.assertEqual(b("Hope all is well.\n"), g.readline(1000))
-        self.assertEqual(b("Bye"), g.readline())
-        self.assertEqual(b(""), g.readline())
+        self.assertEqual(b"H", g.read(1))
+        self.assertEqual(b"ello world,\n", g.readline())
+        self.assertEqual(b"How a", g.readline(5))
+        self.assertEqual(b"", g.readline(0))
+        self.assertEqual(b"re you?\n", g.readline())
+        self.assertEqual(b"Hope all is well.\n", g.readline(1000))
+        self.assertEqual(b"Bye", g.readline())
+        self.assertEqual(b"", g.readline())
 
         # Try readline() first, then read().
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual(b("He"), g.readline(2))
-        self.assertEqual(b("l"), g.read(1))
-        self.assertEqual(b("lo"), g.readline(2))
-        self.assertEqual(b(" world,\n"), g.readline())
+        self.assertEqual(b"He", g.readline(2))
+        self.assertEqual(b"l", g.read(1))
+        self.assertEqual(b"lo", g.readline(2))
+        self.assertEqual(b" world,\n", g.readline())
 
         # Only readline().
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual(b("H"), g.readline(1))
-        self.assertEqual(b("e"), g.readline(1))
-        self.assertEqual(b("llo world,\n"), g.readline())
+        self.assertEqual(b"H", g.readline(1))
+        self.assertEqual(b"e", g.readline(1))
+        self.assertEqual(b"llo world,\n", g.readline())
 
     def test_iterator(self):
         f = GridIn(self.db.fs)
@@ -434,30 +446,30 @@ Bye"""))
         self.assertEqual([], list(g))
 
         f = GridIn(self.db.fs)
-        f.write(b("hello world"))
+        f.write(b"hello world")
         f.close()
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual([b("hello world")], list(g))
-        self.assertEqual(b("hello"), g.read(5))
-        self.assertEqual([b("hello world")], list(g))
-        self.assertEqual(b(" worl"), g.read(5))
+        self.assertEqual([b"hello world"], list(g))
+        self.assertEqual(b"hello", g.read(5))
+        self.assertEqual([b"hello world"], list(g))
+        self.assertEqual(b" worl", g.read(5))
 
         f = GridIn(self.db.fs, chunk_size=2)
-        f.write(b("hello world"))
+        f.write(b"hello world")
         f.close()
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual([b("he"), b("ll"), b("o "),
-                          b("wo"), b("rl"), b("d")], list(g))
+        self.assertEqual([b"he", b"ll", b"o ",
+                          b"wo", b"rl", b"d"], list(g))
 
     def test_read_unaligned_buffer_size(self):
-        in_data = b("This is a text that doesn't "
-                    "quite fit in a single 16-byte chunk.")
+        in_data = (b"This is a text that doesn't "
+                   b"quite fit in a single 16-byte chunk.")
         f = GridIn(self.db.fs, chunkSize=16)
         f.write(in_data)
         f.close()
 
         g = GridOut(self.db.fs, f._id)
-        out_data = b('')
+        out_data = b''
         while 1:
             s = g.read(13)
             if not s:
@@ -467,7 +479,7 @@ Bye"""))
         self.assertEqual(in_data, out_data)
 
     def test_readchunk(self):
-        in_data = b('a') * 10
+        in_data = b'a' * 10
         f = GridIn(self.db.fs, chunkSize=3)
         f.write(in_data)
         f.close()
@@ -486,21 +498,21 @@ Bye"""))
 
     def test_write_unicode(self):
         f = GridIn(self.db.fs)
-        self.assertRaises(TypeError, f.write, u"foo")
+        self.assertRaises(TypeError, f.write, u("foo"))
 
         f = GridIn(self.db.fs, encoding="utf-8")
-        f.write(u"foo")
+        f.write(u("foo"))
         f.close()
 
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual(b("foo"), g.read())
+        self.assertEqual(b"foo", g.read())
 
         f = GridIn(self.db.fs, encoding="iso-8859-1")
-        f.write(u"aé")
+        f.write(u("aé"))
         f.close()
 
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual(u"aé".encode("iso-8859-1"), g.read())
+        self.assertEqual(u("aé").encode("iso-8859-1"), g.read())
 
     def test_set_after_close(self):
         f = GridIn(self.db.fs, _id="foo", bar="baz")
@@ -538,18 +550,13 @@ Bye"""))
         self.assertRaises(AttributeError, getattr, g, "_closed")
 
     def test_context_manager(self):
-        if sys.version_info < (2, 6):
-            raise SkipTest("With statement requires Python >= 2.6")
+        contents = b"Imagine this is some important data..."
 
-        contents = b("Imagine this is some important data...")
-        # Hack around python2.4 an 2.5 not supporting 'with' syntax
-        exec """
-with GridIn(self.db.fs, filename="important") as infile:
-    infile.write(contents)
+        with GridIn(self.db.fs, filename="important") as infile:
+            infile.write(contents)
 
-with GridOut(self.db.fs, infile._id) as outfile:
-    self.assertEqual(contents, outfile.read())
-"""
+        with GridOut(self.db.fs, infile._id) as outfile:
+            self.assertEqual(contents, outfile.read())
 
     def test_prechunked_string(self):
 
@@ -558,7 +565,7 @@ with GridOut(self.db.fs, infile._id) as outfile:
             infile = GridIn(self.db.fs)
             while True:
                 to_write = buf.read(chunk_size)
-                if to_write == b(''):
+                if to_write == b'':
                     break
                 infile.write(to_write)
             infile.close()
@@ -568,7 +575,7 @@ with GridOut(self.db.fs, infile._id) as outfile:
             data = outfile.read()
             self.assertEqual(s, data)
 
-        s = b('x' * DEFAULT_CHUNK_SIZE * 4)
+        s = b'x' * DEFAULT_CHUNK_SIZE * 4
         # Test with default chunk size
         write_me(s, DEFAULT_CHUNK_SIZE)
         # Multiple
@@ -578,47 +585,32 @@ with GridOut(self.db.fs, infile._id) as outfile:
 
     def test_grid_out_lazy_connect(self):
         fs = self.db.fs
-        outfile = GridOut(fs, file_id=-1, _connect=False)
+        outfile = GridOut(fs, file_id=-1)
         self.assertRaises(NoFile, outfile.read)
         self.assertRaises(NoFile, getattr, outfile, 'filename')
 
         infile = GridIn(fs, filename=1)
         infile.close()
 
-        outfile = GridOut(fs, infile._id, _connect=False)
+        outfile = GridOut(fs, infile._id)
         outfile.read()
         outfile.filename
 
-        outfile = GridOut(fs, infile._id, _connect=False)
+        outfile = GridOut(fs, infile._id)
         outfile.readchunk()
 
     def test_grid_in_lazy_connect(self):
-        client = MongoClient('badhost', _connect=False)
+        client = MongoClient('badhost', connect=False,
+                             serverSelectionTimeoutMS=10)
         fs = client.db.fs
         infile = GridIn(fs, file_id=-1, chunk_size=1)
-        self.assertRaises(ConnectionFailure, infile.write, b('data goes here'))
+        self.assertRaises(ConnectionFailure, infile.write, b'data')
         self.assertRaises(ConnectionFailure, infile.close)
 
-    def test_grid_out_cursor_options(self):
-        self.assertRaises(TypeError, GridOutCursor.__init__, self.db.fs, {},
-                                                    tailable=True)
-        self.assertRaises(TypeError, GridOutCursor.__init__, self.db.fs, {},
-                                                    fields={"filename":1})
-
-        cursor = GridOutCursor(self.db.fs, {})
-        min_ms = self.db.fs.files.secondary_acceptable_latency_ms
-        new_ms = cursor._Cursor__secondary_acceptable_latency_ms
-        self.assertEqual(min_ms, new_ms)
-        cursor = GridOutCursor(self.db.fs, {},
-                               secondary_acceptable_latency_ms=100)
-        min_ms = self.db.fs.files.secondary_acceptable_latency_ms
-        new_ms = cursor._Cursor__secondary_acceptable_latency_ms
-        self.assertNotEqual(min_ms, new_ms)
-        cursor_clone = cursor.clone()
-        self.assertEqual(cursor_clone.__dict__, cursor.__dict__)
-
-        self.assertRaises(NotImplementedError, cursor.add_option, 0)
-        self.assertRaises(NotImplementedError, cursor.remove_option, 0)
+    def test_unacknowledged(self):
+        # w=0 is prohibited.
+        with self.assertRaises(ConfigurationError):
+            GridIn(rs_or_single_client(w=0).pymongo_test.fs)
 
 
 if __name__ == "__main__":
