@@ -15,8 +15,10 @@
 """Test the replica_set_connection module."""
 
 import contextlib
+import copy
 import random
 import sys
+import pickle
 
 sys.path[0:0] = [""]
 
@@ -29,7 +31,8 @@ from pymongo.read_preferences import (ReadPreference, MovingAverage,
                                       Primary, PrimaryPreferred,
                                       Secondary, SecondaryPreferred,
                                       Nearest, _ServerMode)
-from pymongo.server_selectors import any_server_selector
+from pymongo.server_description import ServerDescription
+from pymongo.server_selectors import readable_server_selector
 from pymongo.server_type import SERVER_TYPE
 from pymongo.write_concern import WriteConcern
 
@@ -41,10 +44,22 @@ from test import (SkipTest,
                   unittest,
                   db_user,
                   db_pwd)
-from test.utils import single_client, one, wait_until, rs_client
+from test.utils import connected, single_client, one, wait_until, rs_client
 from test.version import Version
 
 
+class TestReadPreferenceObjects(unittest.TestCase):
+    prefs = [Primary(), Secondary(), Nearest(tag_sets=[{'a': 1}, {'b': 2}])]
+
+    def test_pickle(self):
+        for pref in self.prefs:
+            self.assertEqual(pref, pickle.loads(pickle.dumps(pref)))
+
+    def test_copy(self):
+        for pref in self.prefs:
+            self.assertEqual(pref, copy.copy(pref))
+
+
 class TestReadPreferencesBase(TestReplicaSetClientBase):
 
     def setUp(self):
@@ -83,7 +98,7 @@ class TestReadPreferencesBase(TestReplicaSetClientBase):
     def assertReadsFrom(self, expected, **kwargs):
         c = rs_client(**kwargs)
         wait_until(
-            lambda: len(c.nodes) == self.w,
+            lambda: len(c.nodes - c.arbiters) == self.w,
             "discovered all nodes")
 
         used = self.read_from_which_kind(c)
@@ -189,6 +204,33 @@ class TestReadPreferences(TestReadPreferencesBase):
             localthresholdms=666
         ).local_threshold_ms)
 
+        self.assertEqual(0, rs_client(
+            localthresholdms=0
+        ).local_threshold_ms)
+
+        self.assertRaises(ValueError,
+                          rs_client,
+                          localthresholdms=-1)
+
+    def test_zero_latency(self):
+        ping_times = set()
+        # Generate unique ping times.
+        while len(ping_times) < len(self.client.nodes):
+            ping_times.add(random.random())
+        for ping_time, host in zip(ping_times, self.client.nodes):
+            ServerDescription._host_to_round_trip_time[host] = ping_time
+        try:
+            client = connected(
+                rs_client(readPreference='nearest', localThresholdMS=0))
+            wait_until(
+                lambda: client.nodes == self.client.nodes,
+                "discovered all nodes")
+            host = self.read_from_which_host(client)
+            for _ in range(5):
+                self.assertEqual(host, self.read_from_which_host(client))
+        finally:
+            ServerDescription._host_to_round_trip_time.clear()
+
     def test_primary(self):
         self.assertReadsFrom(
             'primary', read_preference=ReadPreference.PRIMARY)
@@ -235,7 +277,8 @@ class TestReadPreferences(TestReadPreferencesBase):
         latencies = ', '.join(
             '%s: %dms' % (server.description.address,
                           server.description.round_trip_time)
-            for server in c._get_topology().select_servers(any_server_selector))
+            for server in c._get_topology().select_servers(
+                readable_server_selector))
 
         self.assertFalse(
             not_used,
