@@ -1,4 +1,4 @@
-# Copyright 2009-2012 10gen, Inc.
+# Copyright 2009-2015 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,6 @@
 """Tests for the Timestamp class."""
 
 import datetime
-import unittest
 import sys
 import copy
 import pickle
@@ -23,13 +22,10 @@ sys.path[0:0] = [""]
 
 from bson.timestamp import Timestamp
 from bson.tz_util import utc
+from test import unittest
 
 
 class TestTimestamp(unittest.TestCase):
-
-    def setUp(self):
-        pass
-
     def test_timestamp(self):
         t = Timestamp(123, 456)
         self.assertEqual(t.time, 123)
